﻿using System;
using System.Diagnostics;
using System.Windows.Forms;

namespace PowerCodeCalculator
{
    using PowerCodeCalculator.Model;
    using PowerCodeCalculator.Service;
    using PowerCodeCalculator.Util;

    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
            Init();
        }

        /// <summary>
        /// 初始化一些数据
        /// </summary>
        private void Init()
        {
            //CodeTimer.Initialize();

            this.cbIsCalcComment.Checked = true;
            //this.cbIsCalcEmptyLine.Checked = false;
            //this.cbIsEnableMultiThread.Checked = false;

            this.cblFileType.SelectedIndex = 0;
            this.txtIgnoreFileType.Text = ".designer.cs,assemblyinfo.cs";
        }

        #region fields


        #endregion

        #region click

        private void btnSelectFolder_Click(object sender, EventArgs e)
        {
            if (fbdCodePath.ShowDialog() == DialogResult.OK)
            {
                this.txtCodePath.Text = fbdCodePath.SelectedPath;
            }
        }

        /// <summary>
        /// 选择代码路径
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void txtCodePath_DoubleClick(object sender, System.EventArgs e)
        {
            if (fbdCodePath.ShowDialog() == DialogResult.OK)
            {
                this.txtCodePath.Text = fbdCodePath.SelectedPath;
            }
        }

        /// <summary>
        /// 单击开始统计代码行数
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCalc_Click(object sender, System.EventArgs e)
        {
            var stopWatch = Stopwatch.StartNew();

            this.rtbOutMsg.Clear();

            var statCondition = new CodeStatisticCondition
            {
                CodeDirectory = this.txtCodePath.Text.Trim(),

                FileTypeFilter = this.cblFileType.SelectedItem.ToString().Trim().ToLower(),
                IgnoreFileType = this.txtIgnoreFileType.Text.Trim().ToLower(),

                IsCalcBrace = this.cbIsCalcBrace.Checked,
                IsCalcEmptyLine = this.cbIsCalcEmptyLine.Checked,
                IsCalcComment = this.cbIsCalcComment.Checked,
                IsEnableMultiThread = this.cbIsEnableMultiThread.Checked,

            };

            try
            {
                this.btnCalc.Text = "处理中...";
                this.btnCalc.Enabled = false;

                //System.Threading.Thread.Sleep(1000 * 1);

                //开始统计代码行数
                var statResult = CodeCounterService.Calc(statCondition);
                SetOutPut(statResult.OutPutInfo);
            }
            finally
            {
                this.btnCalc.Text = "开始统计";
                this.btnCalc.Enabled = true;
            }

            stopWatch.Stop();

            var strElapsed = string.Format("本次{0}统计耗时{1}分{2}秒{3}毫秒",
                statCondition.IsEnableMultiThread ? "多线程" : "单线程",
                stopWatch.Elapsed.Minutes,
                stopWatch.Elapsed.Seconds, stopWatch.Elapsed.Milliseconds);

            SetOutPut(Environment.NewLine);
            SetOutPut(strElapsed);
        }

        private void btnNotePad_Click(object sender, EventArgs e)
        {
            var msg = this.rtbOutMsg.Text;
            if (string.IsNullOrWhiteSpace(msg) == true)
            {
                MessageBox.Show("没有需要显示的内容");
                return;
            }

            //打开记事本 显示完整信息
            NotePadUtil.OpenNotePad(ProcessWindowStyle.Maximized, msg);
        }

        #endregion

        #region util

        private delegate void ShowMessage(string message);

        private void SetOutPut(string feedBack)
        {
            PowerLogger.AppendLog(feedBack); //记录本地文本日志

            if (this.rtbOutMsg.InvokeRequired == false)
            {
                this.rtbOutMsg.Text += feedBack;
                return;
            }


            var func = new ShowMessage(SetOutPut);
            this.BeginInvoke(func, new object[] { feedBack });
        }

        #endregion

    }
}